require 'strict'
local hc = require 'hardoncollider'
local vector = require 'hardoncollider.vector'

p = nil
mouse = nil
shapes = {}
no_game_code = nil
points = {}

local svec
local msg = ""
function love.load()
	hc.init(75)
	hc.setCallbacks(
		function(dt, a,b, sx,sy)
			msg = string.format("start: %q, %q", a.id,b.id)
			svec = vector.new(sx,sy)
		end,
		function(dt, a,b, sx,sy)
			msg = string.format("persist: %q, %q", a.id,b.id)
			if a.id == "mouse" then
				a:move(sx,sy)
			elseif b.id == "mouse" then
				b:move(sx,sy)
			end
			svec = vector.new(sx,sy)
		end,
		function(dt, a,b)
			msg = string.format("stop: %q, %q", a.id,b.id)
			svec = nil
		end)
	mouse = hc.newCircleShape(400,300,20,5)
	mouse.id = "mouse"
	local coords = {}
	for i=1,8 do
		local phi = 2*math.pi * i / 8
		local x,y = math.cos(phi), math.sin(phi)
		local rad = i % 2 == 1 and 90 or 30
		coords[#coords+1] = x * rad
		coords[#coords+1] = y * rad
	end
	shapes[1] = hc.newPolygonShape( unpack(coords) )
	shapes[1]:move(100,300)
	shapes[1].id = "star1"

	shapes[2] = hc.newPolygonShape( unpack(coords) )
	shapes[2]:move(700,300)
	shapes[2].id = "star2"
end

function love.update(dt)
	local v,w = vector.new(love.mouse.getPosition()), vector.new(mouse:centroid())
	mouse:move( dt * 5 * (v - w) )
	mouse:rotate(dt)
	shapes[1]:rotate(dt)
	shapes[2]:rotate(-dt)
	hc.update(dt)
end

function love.draw()
	love.graphics.setColor(255,255,255,100)
	love.graphics.setLine(1)
	for i = 0,800,hc.hash.cell_size do
		love.graphics.line(i,0, i,600)
	end
	for i = 0,600,hc.hash.cell_size do
		love.graphics.line(0,i, 800,i)
	end
	love.graphics.setColor(100,255,150,100)
	for x = 0,800,hc.hash.cell_size do
		for y = 0,600,hc.hash.cell_size do
			local n = hc.hash:getNeighbors('foo', vector.new(x+1,y+1),vector.new(x+2,y+2))
			if #n > 0 then
				love.graphics.rectangle('fill',x,y,hc.hash.cell_size,hc.hash.cell_size)
			end
		end
	end

	love.graphics.setColor(255,255,255,255)
	mouse:draw('fill')
	if p then
		love.graphics.setLine(4)
		love.graphics.setColor(255,255,255,200)
		love.graphics.polygon('line', p:unpack())
		love.graphics.setLine(1)
	end
	love.graphics.setColor(180,200,255,255)
	for i=1,#shapes do
		shapes[i]:draw('fill')
	end

	love.graphics.setColor(255,200,150)
	for i=1,#points-1,2 do
		love.graphics.circle('line', points[i], points[i+1], 5, 3)
	end

	if svec then
		love.graphics.setColor(150,0,0)
		love.graphics.setLine(4)
		local m = vector.new(mouse:centroid())
		love.graphics.line(m.x,m.y, (m+svec):unpack())
	end

	love.graphics.setColor(255,255,255)
	love.graphics.print(string.format("%d", love.timer.getFPS()), 10, 10)
	love.graphics.print(msg, 10,30)
end

function love.mousereleased(x,y,btn)
	pcall(function()
		if btn == 'l' then
			points[#points+1] = x
			points[#points+1] = y
			if #points >= 6 then
				p = Polygon(unpack(points))
			end
		elseif btn == 'r' and #points >= 6 then
			shapes[#shapes+1] = hc.newPolygonShape(unpack(points))
			shapes[#shapes].id = string.format("shape %d", #shapes)
			points = {}
			p = nil
		end
	end)
end
